@@ -5,7 +5,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"net/url"
 	"os"
@@ -24,7 +23,7 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
 	"github.com/spf13/viper"
-	"golang.org/x/crypto/ssh/terminal"
+	"golang.org/x/term"
 	"gopkg.in/yaml.v2"
 )
 
@@ -50,39 +49,7 @@ var Stdout io.Writer = os.Stdout
 // otherwise it defaults to `os.Stderr`.
 var Stderr io.Writer = os.Stderr
 
-// Ugh, see https://github.com/spf13/cobra/issues/836
-var usageTemplate = `Usage:{{if .Runnable}}
-  {{.UseLine}}{{end}}{{if .HasAvailableSubCommands}}
-  {{.CommandPath}} [command]{{end}}{{if gt (len .Aliases) 0}}
-
-Aliases:
-  {{.NameAndAliases}}{{end}}{{if .HasExample}}
-
-Examples:
-{{.Example}}{{end}}{{if (not .Parent)}}{{if (gt (len .Commands) 9)}}
-
-Available API Commands:{{range .Commands}}{{if (not (or (eq .Name "help") (eq .Name "get") (eq .Name "put") (eq .Name "post") (eq .Name "patch") (eq .Name "delete") (eq .Name "head") (eq .Name "options") (eq .Name "cert") (eq .Name "api") (eq .Name "links") (eq .Name "edit") (eq .Name "completion") (eq .Name "auth-header")))}}
-  {{rpad .Name .NamePadding }} {{.Short}}{{end}}{{end}}{{end}}{{if .HasAvailableSubCommands}}
-
-Generic Commands:{{range .Commands}}{{if (or (eq .Name "help") (eq .Name "get") (eq .Name "put") (eq .Name "post") (eq .Name "patch") (eq .Name "delete") (eq .Name "head") (eq .Name "options") (eq .Name "cert") (eq .Name "api") (eq .Name "links") (eq .Name "edit") (eq .Name "completion") (eq .Name "auth-header"))}}
-  {{rpad .Name .NamePadding }} {{.Short}}{{end}}{{end}}{{end}}{{else}}{{if .HasAvailableSubCommands}}
-
-Available Commands:{{range .Commands}}{{if (or .IsAvailableCommand (eq .Name "help"))}}
-  {{rpad .Name .NamePadding }} {{.Short}}{{end}}{{end}}{{end}}{{end}}{{if .HasAvailableLocalFlags}}
-
-Flags:
-{{.LocalFlags.FlagUsages | trimTrailingWhitespaces}}{{end}}{{if .HasAvailableInheritedFlags}}
-
-Global Flags:
-{{.InheritedFlags.FlagUsages | trimTrailingWhitespaces}}{{end}}{{if .HasHelpSubCommands}}
-
-Additional help topics:{{range .Commands}}{{if .IsAdditionalHelpTopicCommand}}
-  {{rpad .CommandPath .CommandPathPadding}} {{.Short}}{{end}}{{end}}{{end}}{{if .HasAvailableSubCommands}}
-
-Use "{{.CommandPath}} [command] --help" for more information about a command.{{end}}
-`
-
-var tty bool
+var useColor bool
 var au aurora.Aurora
 
 // Keeps track of currently selected API for shell completions
@@ -149,7 +116,17 @@ func completeCurrentConfig(cmd *cobra.Command, args []string, toComplete string,
 		for _, cmd := range Root.Commands() {
 			if cmd.Use == currentConfig.name {
 				// This is the matching command. Load the URL and check each operation.
-				api, _ := Load(currentConfig.Base, cmd)
+				currentBase := currentConfig.Base
+				currentProfile := currentConfig.Profiles[viper.GetString("rsh-profile")]
+				if currentProfile == nil {
+					if viper.GetString("rsh-profile") != "default" {
+						panic("invalid profile " + viper.GetString("rsh-profile"))
+					}
+				}
+				if currentProfile != nil && currentProfile.Base != "" {
+					currentBase = currentProfile.Base
+				}
+				api, _ := Load(currentBase, cmd)
 				for _, op := range api.Operations {
 					if op.Method != method {
 						// We only care about operations which match the currently selected
@@ -213,37 +190,37 @@ func Init(name string, version string) {
 
 	// Reset registries.
 	authHandlers = map[string]AuthHandler{}
-	contentTypes = []contentTypeEntry{}
+	contentTypes = map[string]contentTypeEntry{}
 	encodings = map[string]ContentEncoding{}
 	linkParsers = []LinkParser{}
 	loaders = []Loader{}
 
 	// Determine if we are using a TTY or colored output is forced-on.
-	tty = false
-	if isatty.IsTerminal(os.Stdout.Fd()) || isatty.IsCygwinTerminal(os.Stdout.Fd()) || viper.GetBool("color") {
+	tty := false
+	if isatty.IsTerminal(os.Stdout.Fd()) || isatty.IsCygwinTerminal(os.Stdout.Fd()) || viper.GetBool("tty") {
 		tty = true
 	}
 
-	if viper.GetBool("nocolor") {
-		// If forced off, ignore all of the above!
-		tty = false
+	useColor = false
+	if viper.GetBool("color") || (tty && !viper.GetBool("nocolor")) {
+		useColor = true
 	}
 
-	if tty {
+	if useColor {
 		// Support colored output across operating systems.
 		Stdout = colorable.NewColorableStdout()
 		Stderr = colorable.NewColorableStderr()
 	}
 
-	au = aurora.NewAurora(tty)
+	au = aurora.NewAurora(useColor)
 
-	Formatter = NewDefaultFormatter(tty)
+	Formatter = NewDefaultFormatter(tty, useColor)
 
 	cobra.AddTemplateFunc("highlight", func(s string) string {
 		// Highlighting is expensive, so only do this when the user actually asks
 		// for help via this template func and a custom help template.
 		if tty {
-			w, _, err := terminal.GetSize(0)
+			w, _, err := term.GetSize(0)
 			if err != nil {
 				// Default to standard terminal size
 				w = 80
@@ -278,12 +255,16 @@ func Init(name string, version string) {
 			generic(http.MethodGet, args[0], args[1:])
 		},
 	}
-	Root.SetUsageTemplate(usageTemplate)
+	Root.AddGroup(
+		&cobra.Group{ID: "api", Title: "Available API Commands:"},
+		&cobra.Group{ID: "generic", Title: "Generic Commands:"},
+	)
 	Root.SetHelpTemplate(`{{with (or .Long .Short)}}{{. | trimTrailingWhitespaces | highlight}}
 
 {{end}}{{if or .Runnable .HasSubCommands}}{{.UsageString}}{{end}}`)
 
 	head := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "head uri",
 		Short:             "Head a URI",
 		Long:              "Perform an HTTP HEAD on the given URI",
@@ -296,6 +277,7 @@ func Init(name string, version string) {
 	Root.AddCommand(head)
 
 	options := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "options uri",
 		Short:             "Options a URI",
 		Long:              "Perform an HTTP OPTIONS on the given URI",
@@ -308,6 +290,7 @@ func Init(name string, version string) {
 	Root.AddCommand(options)
 
 	get := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "get uri",
 		Short:             "Get a URI",
 		Long:              "Perform an HTTP GET on the given URI",
@@ -320,6 +303,7 @@ func Init(name string, version string) {
 	Root.AddCommand(get)
 
 	post := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "post uri [body...]",
 		Short:             "Post a URI",
 		Long:              "Perform an HTTP POST on the given URI",
@@ -332,6 +316,7 @@ func Init(name string, version string) {
 	Root.AddCommand(post)
 
 	put := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "put uri [body...]",
 		Short:             "Put a URI",
 		Long:              "Perform an HTTP PUT on the given URI",
@@ -344,6 +329,7 @@ func Init(name string, version string) {
 	Root.AddCommand(put)
 
 	patch := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "patch uri [body...]",
 		Short:             "Patch a URI",
 		Long:              "Perform an HTTP PATCH on the given URI",
@@ -356,6 +342,7 @@ func Init(name string, version string) {
 	Root.AddCommand(patch)
 
 	delete := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "delete uri [body...]",
 		Short:             "Delete a URI",
 		Long:              "Perform an HTTP DELETE on the given URI",
@@ -371,6 +358,7 @@ func Init(name string, version string) {
 	var noPrompt *bool
 	var editFormat *string
 	edit := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "edit uri [-i] [body...]",
 		Short:             "Edit a resource by URI",
 		Long:              "Convenience function which combines a GET, edit, and PUT operation into one command",
@@ -393,9 +381,10 @@ func Init(name string, version string) {
 	Root.AddCommand(edit)
 
 	authHeader := &cobra.Command{
-		Use:   "auth-header uri",
-		Short: "Get an auth header for a given API",
-		Long:  "Get an OAuth2 bearer token in an Authorization header capable of being passed to other commands. Uses a cached token when possible, renewing as needed if it has expired.",
+		GroupID: "generic",
+		Use:     "auth-header uri",
+		Short:   "Get an auth header for a given API",
+		Long:    "Get an OAuth2 bearer token in an Authorization header capable of being passed to other commands. Uses a cached token when possible, renewing as needed if it has expired.",
 		Example: fmt.Sprintf(`  # Using API short name
   $ %s auth-header my-api
 
@@ -411,16 +400,16 @@ func Init(name string, version string) {
 			name, config := findAPI(addr)
 
 			if config == nil {
-				return fmt.Errorf("No matched API for URL %s", args[0])
+				return fmt.Errorf("no matched API for URL %s", args[0])
 			}
 
 			profile := config.Profiles[viper.GetString("rsh-profile")]
 			if profile == nil {
-				return fmt.Errorf("Invalid profile %s", viper.GetString("rsh-profile"))
+				return fmt.Errorf("invalid profile %s", viper.GetString("rsh-profile"))
 			}
 
 			if profile.Auth == nil || profile.Auth.Name == "" {
-				return fmt.Errorf("No auth set up for API")
+				return fmt.Errorf("no auth set up for API")
 			}
 
 			if auth, ok := authHandlers[profile.Auth.Name]; ok {
@@ -437,6 +426,7 @@ func Init(name string, version string) {
 	Root.AddCommand(authHeader)
 
 	cert := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "cert uri",
 		Short:             "Get cert info",
 		Long:              "Get TLS certificate information including expiration date",
@@ -459,12 +449,12 @@ func Init(name string, version string) {
 			}
 
 			chains := conn.ConnectionState().VerifiedChains
-			if chains != nil && len(chains) > 0 && len(chains[0]) > 0 {
+			if len(chains) > 0 && len(chains[0]) > 0 {
 				// The first cert in the first chain should represent the domain.
 				c := chains[0][0]
 
 				expiresRelative := ""
-				days := c.NotAfter.Sub(time.Now()).Hours() / 24
+				days := time.Until(c.NotAfter).Hours() / 24
 				if days > 0 {
 					expiresRelative = fmt.Sprintf("in %.1f days", days)
 				} else {
@@ -489,6 +479,7 @@ Not after (expires): %s (%s)
 	Root.AddCommand(cert)
 
 	linkCmd := &cobra.Command{
+		GroupID:           "generic",
 		Use:               "links uri [rel1 rel2...]",
 		Short:             "Get link relations from the given URI, with optional filtering",
 		Long:              "Returns a list of resolved references to the link relations after making an HTTP GET request to the given URI. Additional arguments filter down the set of returned relationship names.",
@@ -506,9 +497,7 @@ Not after (expires): %s (%s)
 			if len(args) > 1 {
 				tmp := []*Link{}
 				for _, rel := range args[1:] {
-					for _, link := range resp.Links[rel] {
-						tmp = append(tmp, link)
-					}
+					tmp = append(tmp, resp.Links[rel]...)
 				}
 				output = tmp
 			}
@@ -518,7 +507,7 @@ Not after (expires): %s (%s)
 				panic(err)
 			}
 
-			if tty {
+			if useColor {
 				encoded, err = Highlight("json", encoded)
 				if err != nil {
 					panic(err)
@@ -539,8 +528,8 @@ Not after (expires): %s (%s)
 	GlobalFlags.BoolP("help", "h", false, "")
 
 	AddGlobalFlag("rsh-verbose", "v", "Enable verbose log output", false, false)
-	AddGlobalFlag("rsh-output-format", "o", "Output format [auto, json, yaml]", "auto", false)
-	AddGlobalFlag("rsh-filter", "f", "Filter / project results using JMESPath Plus", "", false)
+	AddGlobalFlag("rsh-output-format", "o", "Output format [auto, json, table, ...]", "auto", false)
+	AddGlobalFlag("rsh-filter", "f", "Filter / project results using shorthand query", "", false)
 	AddGlobalFlag("rsh-raw", "r", "Output result of query as raw rather than an escaped JSON string or list", false, false)
 	AddGlobalFlag("rsh-server", "s", "Override scheme://server:port for an API", "", false)
 	AddGlobalFlag("rsh-header", "H", "Add custom header", []string{}, true)
@@ -552,7 +541,6 @@ Not after (expires): %s (%s)
 	AddGlobalFlag("rsh-client-cert", "", "Path to a PEM encoded client certificate", "", false)
 	AddGlobalFlag("rsh-client-key", "", "Path to a PEM encoded private key", "", false)
 	AddGlobalFlag("rsh-ca-cert", "", "Path to a PEM encoded CA cert", "", false)
-	AddGlobalFlag("rsh-table", "t", "Enable table formatted output for array of objects", false, false)
 
 	Root.RegisterFlagCompletionFunc("rsh-output-format", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return []string{"auto", "json", "yaml"}, cobra.ShellCompDirectiveNoFileComp
@@ -621,7 +609,7 @@ func initCache(appName string) {
 	// cli.Cache.SaveConfig() to write new values.
 	filename := path.Join(viper.GetString("config-directory"), "cache.json")
 	if _, err := os.Stat(filename); os.IsNotExist(err) {
-		if err := ioutil.WriteFile(filename, []byte("{}"), 0600); err != nil {
+		if err := os.WriteFile(filename, []byte("{}"), 0600); err != nil {
 			panic(err)
 		}
 	}
@@ -637,12 +625,14 @@ func Defaults() {
 	AddEncoding("br", &BrotliEncoding{})
 
 	// Register content type marshallers
-	AddContentType("application/cbor", 0.9, &CBOR{})
-	AddContentType("application/msgpack", 0.8, &MsgPack{})
-	AddContentType("application/ion", 0.6, &Ion{})
-	AddContentType("application/json", 0.5, &JSON{})
-	AddContentType("application/yaml", 0.5, &YAML{})
-	AddContentType("text/*", 0.2, &Text{})
+	AddContentType("cbor", "application/cbor", 0.9, &CBOR{})
+	AddContentType("msgpack", "application/msgpack", 0.8, &MsgPack{})
+	AddContentType("ion", "application/ion", 0.6, &Ion{})
+	AddContentType("json", "application/json", 0.5, &JSON{})
+	AddContentType("yaml", "application/yaml", 0.5, &YAML{})
+	AddContentType("text", "text/*", 0.2, &Text{})
+	AddContentType("table", "", -1, &Table{})
+	AddContentType("readable", "", -1, &Readable{})
 
 	// Add link relation parsers
 	AddLinkParser(&LinkHeaderParser{})
@@ -652,6 +642,7 @@ func Defaults() {
 
 	// Register auth schemes
 	AddAuth("http-basic", &BasicAuth{})
+	AddAuth("external-tool", &ExternalToolAuth{})
 }
 
 // Run the CLI! Parse arguments, make requests, print responses.
@@ -668,6 +659,13 @@ func Run() {
 		}
 	}
 
+	if os.Getenv("COLOR") != "" {
+		viper.Set("color", true)
+	}
+	if os.Getenv("NOCOLOR") != "" {
+		viper.Set("nocolor", true)
+	}
+
 	// Because we may be doing HTTP calls before cobra has parsed the flags
 	// we parse the GlobalFlags here and already set some config values
 	// to ensure they are available
@@ -700,6 +698,7 @@ func Run() {
 	if headers, _ := GlobalFlags.GetStringSlice("rsh-header"); len(headers) > 0 {
 		viper.Set("rsh-header", headers)
 	}
+	profile, _ := GlobalFlags.GetString("rsh-profile")
 
 	// Now that global flags are parsed we can enable verbose mode if requested.
 	if viper.GetBool("rsh-verbose") {
@@ -725,7 +724,17 @@ func Run() {
 				currentConfig = cfg
 				for _, cmd := range Root.Commands() {
 					if cmd.Use == apiName {
-						if _, err := Load(cfg.Base, cmd); err != nil {
+						currentBase := cfg.Base
+						currentProfile := cfg.Profiles[profile]
+						if currentProfile == nil {
+							if profile != "default" {
+								panic("invalid profile " + profile)
+							}
+						}
+						if currentProfile != nil && currentProfile.Base != "" {
+							currentBase = currentProfile.Base
+						}
+						if _, err := Load(currentBase, cmd); err != nil {
 							panic(err)
 						}
 						loaded = true
